///File : HangMan.cpp
//------------------------------------------------------------------------------
/*
    This file is part of HangMan.

    HangMan is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    HangMan is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with HangMan.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "HangMan.hpp"
//------------------------------------------------------------------------------

HangMan::HangMan() : LetterGameBase()
{
    m_tryFirstLastLetters = true;
    m_nbSucceededWords = 0;
    m_nbPlayedWords = 0;
}
//------------------------------------------------------------------------------
void HangMan::SetTryFirstLastLetters(const bool &tryLetters)
{
    m_tryFirstLastLetters = tryLetters;
}
//------------------------------------------------------------------------------
bool HangMan::IsTryFirstLastLetters() const
{
    return m_tryFirstLastLetters;
}
//------------------------------------------------------------------------------
int HangMan::GetNbSucceededWords() const
{
    return m_nbSucceededWords;
}
//------------------------------------------------------------------------------
int HangMan::GetNbPlayedWords() const
{
    return m_nbPlayedWords;
}
//------------------------------------------------------------------------------
void HangMan::NewWord()
{
    m_currentWordResult = m_wordsTbl[Hasard(0, m_wordsTbl.size() -1)];
    m_triedLetters = _T("");
    m_currentWord = _T("");
    for(int i = 0; i < m_currentWordResult.Len(); i++)
    {
        m_currentWord << UNKNOWN_LETTER;
    }

    if(IsTryFirstLastLetters() && m_currentWordResult.Len() > 2)
    {
        // Les lettres du débuts et fin...
        TryLetter(m_currentWordResult[0]);
        TryLetter(m_currentWordResult[m_currentWordResult.Len() - 1]);
    }

    // On teste les lettres que l'utilisataur ne peut pas essayer...
    wxString everyUserLetter = GetEveryUserLetter();
    for(int i = 0; i < m_currentWordResult.Len(); i++)
    {
        if(everyUserLetter.Find(m_currentWordResult[i]) == wxNOT_FOUND)
        {
            m_currentWord[i] = m_currentWordResult[i];
        }
    }

    m_chanceUsed = 0;
    m_nbPlayedWords++;
}

//-----------------------------------------------------------------------------
bool HangMan::TryLetters(const wxString &letters)
{
    bool isLetterInWord = false;

    for(int i = 0; i < m_currentWordResult.Len(); i++)
    {
        if(letters.Find(m_currentWordResult[i]) != wxNOT_FOUND)
        {
            m_currentWord[i] = m_currentWordResult[i];
            isLetterInWord = true;
        }
    }

    return isLetterInWord;
}
//-----------------------------------------------------------------------------
bool HangMan::TryLetter(const wxChar &letter)
{
    bool isLetterInWord = TryLetters(GetLettersFor(letter));

    if(!isLetterInWord)
    {
        m_chanceUsed++;
    }
    else
    {
        if(IsWon())
        {
            m_nbSucceededWords++;
        }
    }

    wxString str;
    str << letter;
    m_triedLetters << str.Upper()[0];

    return isLetterInWord;
}

//-----------------------------------------------------------------------------
bool HangMan::IsWon() const
{
    return (m_currentWordResult == m_currentWord);
}
//-----------------------------------------------------------------------------
bool HangMan::IsLost() const
{
    return (m_chanceUsed >= HANGMAN_NB_CHANCE);
}
//-----------------------------------------------------------------------------

wxString HangMan::GetTriedLetters() const
{
    return m_triedLetters;
}
//-----------------------------------------------------------------------------
wxString HangMan::GetCurrentWordResult() const
{
    return m_currentWordResult;
}
//-----------------------------------------------------------------------------
wxString HangMan::GetCurrentWord() const
{
    return m_currentWord;
}
//-----------------------------------------------------------------------------

int HangMan::GetChanceLeft() const
{
    return HANGMAN_NB_CHANCE - m_chanceUsed;
}
//-----------------------------------------------------------------------------
int HangMan::GetChanceUsed() const
{
    return m_chanceUsed;
}
//-----------------------------------------------------------------------------
