///File : LongestWord.hpp
//------------------------------------------------------------------------------
/*
    This file is part of LongestWord.

    LongestWord is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    LongestWord is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with LongestWord.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
//------------------------------------------------------------------------------
#include <wx/sound.h>

//------------------------------------------------------------------------------
#include "config.hpp"
#include "LetterGameBase.hpp"

//------------------------------------------------------------------------------

#if not defined HANG_MAN_HPP_LETTER_GAMES
#define HANG_MAN_HPP_LETTER_GAMES
//------------------------------------------------------------------------------

#define HANGMAN_NB_CHANCE 7

#define HASARD_LETTERS_NB 9

#if defined(__WXMSW__)
    #define UNKNOWN_LETTER _T("*")
#else
    #define UNKNOWN_LETTER _T("~")
#endif

class LongestWord : public LetterGameBase
{
public:
    LongestWord();

    wxChar GetOneLetterHasard(const bool &isVoyelle = true, const bool &isConsonne = true) const;
    void NewLetters();

    bool WordExists(const wxString &word) const;
    bool WordUsesCurrentLetters(const wxString &word, wxString *unusedLetters = NULL) const;

    std::vector<wxString> GetLongestWordsForCurrentLetters(const bool &longestOnly = true, const bool &onlyOne = false) const;

    wxString GetCurrentLetters() const;
    wxString GetCurrentWord() const;
    void SetCurrentWord(const wxString &word);

    bool Correct(int *userPointPtr = NULL, int *pcPointPtr = NULL, std::vector<wxString> *longestWordList = NULL);

    int GetUserScore() const;
    int GetPcScore() const;

    int GetTimeLeft() const;

    bool IsInGame() const;

protected:
    wxString m_currentLetters;

    wxString m_currentWord;

    int m_timeLeft, m_userScore, m_pcScore;

    bool m_isInGame;
};

#endif //HANG_MAN_HPP_LETTER_GAMES
